@@ -413,6 +413,7 @@ async def test_delete_data(c, s, a, b):
 async def test_delete(c, s, a):
     x = c.submit(inc, 1)
     await x
+    assert x.key in s.tasks
     assert x.key in a.data
 
     await c._cancel(x)
@@ -422,6 +423,10 @@ async def test_delete(c, s, a):
         await asyncio.sleep(0.01)
         assert time() < start + 5
 
+    assert x.key not in s.tasks
+
+    s.report_on_key(key=x.key)
+
 
 @gen_cluster()
 async def test_filtered_communication(s, a, b):
@@ -1723,6 +1728,18 @@ async def test_dashboard_address():
     assert s.services["dashboard"].port
     await s.close()
 
+    s = await Scheduler(dashboard_address="127.0.0.1:8901,127.0.0.1:8902", port=0)
+    assert s.services["dashboard"].port == 8901
+    await s.close()
+
+    s = await Scheduler(dashboard_address=":8901,:8902", port=0)
+    assert s.services["dashboard"].port == 8901
+    await s.close()
+
+    s = await Scheduler(dashboard_address=[8901, 8902], port=0)
+    assert s.services["dashboard"].port == 8901
+    await s.close()
+
 
 @gen_cluster(client=True)
 async def test_adaptive_target(c, s, a, b):
@@ -2038,7 +2055,6 @@ async def test_gather_no_workers(c, s, a, b):
     assert list(res["keys"]) == ["x"]
 
 
-@pytest.mark.flaky(reruns=10, reruns_delay=5, condition=MACOS)
 @gen_cluster(client=True, client_kwargs={"direct_to_workers": False})
 async def test_gather_allow_worker_reconnect(c, s, a, b):
     """
@@ -2057,9 +2073,10 @@ async def test_gather_allow_worker_reconnect(c, s, a, b):
     def inc_slow(x):
         # Once the graph below is rescheduled this computation runs again. We
         # need to sleep for at least 0.5 seconds to give the worker a chance to
-        # reconnect (Heartbeat timing)
+        # reconnect (Heartbeat timing). In slow CI situations, the actual
+        # reconnect might take a bit longer, therefore wait more
         if x in already_calculated:
-            time.sleep(1)
+            time.sleep(2)
         already_calculated.append(x)
         return x + 1
 
@@ -2071,18 +2088,16 @@ async def test_gather_allow_worker_reconnect(c, s, a, b):
 
     z = c.submit(reducer, x, y)
 
-    s.rpc = await FlakyConnectionPool(failing_connections=4)
+    s.rpc = await FlakyConnectionPool(failing_connections=1)
 
-    with dask.config.set(
-        {"distributed.comm.retry.delay_min": 0.5, "distributed.comm.retry.count": 3}
-    ):
+    # This behaviour is independent of retries. Remove them to reduce complexity
+    # of this setup
+    with dask.config.set({"distributed.comm.retry.count": 0}):
         with captured_logger(
             logging.getLogger("distributed.scheduler")
         ) as sched_logger, captured_logger(
             logging.getLogger("distributed.client")
-        ) as client_logger, captured_logger(
-            logging.getLogger("distributed.utils_comm")
-        ) as utils_comm_logger:
+        ) as client_logger:
             # Gather using the client (as an ordinary user would)
             # Upon a missing key, the client will reschedule the computations
             res = await c.gather(z)
@@ -2091,14 +2106,11 @@ async def test_gather_allow_worker_reconnect(c, s, a, b):
 
     sched_logger = sched_logger.getvalue()
     client_logger = client_logger.getvalue()
-    utils_comm_logger = utils_comm_logger.getvalue()
 
     # Ensure that the communication was done via the scheduler, i.e. we actually hit a
     # bad connection
     assert s.rpc.cnn_count > 0
 
-    assert "Retrying get_data_from_worker after exception" in utils_comm_logger
-
     # The reducer task was actually not found upon first collection. The client will
     # reschedule the graph
     assert "Couldn't gather 1 keys, rescheduling" in client_logger
@@ -2117,7 +2129,6 @@ async def test_gather_allow_worker_reconnect(c, s, a, b):
     ]
     assert len(transitions_to_processing) == 1
 
-    starts = []
     finish_processing_transitions = 0
     for transition in s.transition_log:
         key, start, finish, recommendations, timestamp = transition
@@ -2463,3 +2474,23 @@ async def test_memory_is_none(c, s):
             assert s.memory.unmanaged == 0
             assert s.memory.unmanaged_old == 0
             assert s.memory.unmanaged_recent == 0
+
+
+@gen_cluster()
+async def test_close_scheduler__close_workers_Worker(s, a, b):
+    with captured_logger("distributed.comm", level=logging.DEBUG) as log:
+        await s.close(close_workers=True)
+        while not a.status == Status.closed:
+            await asyncio.sleep(0.05)
+    log = log.getvalue()
+    assert "retry" not in log
+
+
+@gen_cluster(Worker=Nanny)
+async def test_close_scheduler__close_workers_Nanny(s, a, b):
+    with captured_logger("distributed.comm", level=logging.DEBUG) as log:
+        await s.close(close_workers=True)
+        while not a.status == Status.closed:
+            await asyncio.sleep(0.05)
+    log = log.getvalue()
+    assert "retry" not in log
