@@ -1,61 +1,60 @@
-from concurrent.futures import ThreadPoolExecutor
+import asyncio
 import importlib
 import logging
-from numbers import Number
-from operator import add
 import os
-import psutil
 import sys
-from time import sleep
 import threading
 import traceback
+from concurrent.futures import ThreadPoolExecutor
+from numbers import Number
+from operator import add
+from time import sleep
 from unittest import mock
-import asyncio
+
+import psutil
+import pytest
+from tlz import first, pluck, sliding_window
 
 import dask
 from dask import delayed
-from dask.utils import format_bytes
 from dask.system import CPU_COUNT
-import pytest
-from tlz import pluck, sliding_window, first
+from dask.utils import format_bytes
 
 from distributed import (
     Client,
     Nanny,
-    get_client,
-    default_client,
-    get_worker,
     Reschedule,
+    default_client,
+    get_client,
+    get_worker,
     wait,
 )
-from distributed.diagnostics.plugin import PipInstall
 from distributed.compatibility import MACOS, WINDOWS
-from distributed.core import rpc, CommClosedError, Status
-from distributed.scheduler import Scheduler
+from distributed.core import CommClosedError, Status, rpc
+from distributed.diagnostics.plugin import PipInstall
 from distributed.metrics import time
-from distributed.worker import Worker, error_message, logger, parse_memory_limit
-from distributed.utils import tmpfile, TimeoutError
+from distributed.scheduler import Scheduler
+from distributed.utils import TimeoutError, tmpfile
 from distributed.utils_test import (  # noqa: F401
-    cleanup,
-    inc,
-    mul,
-    gen_cluster,
-    div,
-    dec,
-    slowinc,
-    gen_test,
-    captured_logger,
-)
-from distributed.utils_test import (  # noqa: F401
-    client,
-    loop,
-    nodebug,
-    cluster_fixture,
-    s,
+    TaskStateMetadataPlugin,
     a,
     b,
-    TaskStateMetadataPlugin,
+    captured_logger,
+    cleanup,
+    client,
+    cluster_fixture,
+    dec,
+    div,
+    gen_cluster,
+    gen_test,
+    inc,
+    loop,
+    mul,
+    nodebug,
+    s,
+    slowinc,
 )
+from distributed.worker import Worker, error_message, logger, parse_memory_limit, weight
 
 
 @pytest.mark.asyncio
@@ -445,21 +444,16 @@ async def test_spill_to_disk(c, s):
 
     assert set(w.data) == {x.key, y.key}
     assert set(w.data.memory) == {x.key, y.key}
-    assert set(w.data.fast) == set(w.data.memory)
 
     z = c.submit(np.random.randint, 0, 255, size=500, dtype="u1", key="z")
     await wait(z)
     assert set(w.data) == {x.key, y.key, z.key}
     assert set(w.data.memory) == {y.key, z.key}
-    assert set(w.data.disk) == {x.key} or set(w.data.slow) == {x.key, y.key}
-    assert set(w.data.fast) == set(w.data.memory)
-    assert set(w.data.slow) == set(w.data.disk)
+    assert set(w.data.disk) == {x.key}
 
     await x
     assert set(w.data.memory) == {x.key, z.key}
-    assert set(w.data.disk) == {y.key} or set(w.data.slow) == {x.key, y.key}
-    assert set(w.data.fast) == set(w.data.memory)
-    assert set(w.data.slow) == set(w.data.disk)
+    assert set(w.data.disk) == {y.key}
     await w.close()
 
 
@@ -525,7 +519,6 @@ async def test_spill_by_default(c, s, w):
     y = c.persist(x)
     await wait(y)
     assert len(w.data.disk)  # something is on disk
-    del x, y
 
 
 @gen_cluster(nthreads=[("127.0.0.1", 1)], worker_kwargs={"reconnect": False})
@@ -1794,3 +1787,39 @@ async def test_story(c, s, w):
     ts = w.tasks[future.key]
     assert ts.state in str(w.story(ts))
     assert w.story(ts) == w.story(ts.key)
+
+
+@gen_cluster(client=True)
+async def test_story_with_deps(c, s, a, b):
+    """
+    Assert that the structure of the story does not change unintentionally and
+    expected subfields are actually filled
+    """
+    futures = c.map(inc, range(10), workers=[a.address])
+    res = c.submit(sum, futures, workers=[b.address])
+    await res
+    key = res.key
+
+    story = a.story(key)
+    assert story == []
+    story = b.story(key)
+
+    expected_story = [
+        (key, "new"),
+        (key, "new", "waiting"),
+        (
+            "gather-dependencies",
+            key,
+            {fut.key for fut in futures},
+        ),
+        (key, "waiting", "ready"),
+        (key, "ready", "executing"),
+        (key, "put-in-memory"),
+        (key, "executing", "memory"),
+    ]
+    assert story == expected_story
+
+
+def test_weight_deprecated():
+    with pytest.warns(DeprecationWarning):
+        weight("foo", "bar")
